@@ -89,7 +89,7 @@ FlintAllTermsList::next()
 	cursor->next();
 	if (cursor->after_end()) {
 	    current_term = "";
-	    break;
+	    RETURN(NULL);
 	}
 
 	const char *p = cursor->current_key.data();
@@ -98,18 +98,18 @@ FlintAllTermsList::next()
 	    throw Xapian::DatabaseCorruptError("PostList table key has unexpected format");
 	}
 
-	if (!startswith(current_term, prefix)) {
-	    // We've reached the end of the prefixed terms.
-	    cursor->to_end();
-	    current_term = "";
-	    break;
-	}
-
 	// If this key is for the first chunk of a postlist, we're done.
 	// Otherwise we need to skip past continuation chunks until we find the
 	// first chunk of the next postlist.
 	if (p == pend) break;
     }
+
+    if (!startswith(current_term, prefix)) {
+	// We've reached the end of the prefixed terms.
+	cursor->to_end();
+	current_term = "";
+    }
+
     RETURN(NULL);
 }
 
