@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2007,2009 Olly Betts
  * Copyright 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -58,7 +58,7 @@ class FlintPostListTable : public FlintTable {
 	 *  @param readonly_      - whether to open the table for read only
 	 *                          access.
 	 */
-	FlintPostListTable(string path_, bool readonly_)
+	FlintPostListTable(const string & path_, bool readonly_)
 	    : FlintTable(path_ + "/postlist.", readonly_) { }
 
 	/// Merge added, removed, and changed entries.
@@ -111,7 +111,7 @@ class FlintPostList : public LeafPostList {
 	Xapian::Internal::RefCntPtr<const FlintDatabase> this_db;
 
 	/// The termname for this postlist.
-	string tname;
+	string term;
 
 	/// Whether we've started reading the list yet.
 	bool have_started;
@@ -207,7 +207,7 @@ class FlintPostList : public LeafPostList {
     public:
 	/// Default constructor.
 	FlintPostList(Xapian::Internal::RefCntPtr<const FlintDatabase> this_db_,
-		      const string & tname);
+		      const string & term_);
 
 	/// Destructor.
 	~FlintPostList();
@@ -256,7 +256,7 @@ class FlintPostList : public LeafPostList {
 	/// Read the number of entries and the collection frequency.
 	static void read_number_of_entries(const char ** posptr,
 					   const char * end,
-					   Xapian::termcount * number_of_entries_ptr,
+					   Xapian::doccount * number_of_entries_ptr,
 					   Xapian::termcount * collection_freq_ptr);
 };
 
