@@ -1571,6 +1571,20 @@ FlintTable::FlintTable(const string & path_, bool readonly_,
 	      compress_strategy_ << ", " << lazy_);
 }
 
+bool
+FlintTable::really_empty() const
+{
+    if (handle < 0) {
+	if (handle == -2) {
+	    throw Xapian::DatabaseError("Database has been closed");
+	}
+	return true;
+    }
+    FlintCursor cur(const_cast<FlintTable*>(this));
+    cur.find_entry(string());
+    return !cur.next();
+}
+
 void
 FlintTable::lazy_alloc_deflate_zstream() const {
     if (usual(deflate_zstream)) {
