@@ -265,6 +265,9 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	/// Assignment not allowed
         FlintTable & operator=(const FlintTable &);
 
+	/// Return true if there are no entries in the table.
+	bool really_empty() const;
+
     public:
 	/** Create a new Btree object.
 	 *
@@ -517,12 +520,29 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 *
 	 *  The count does not include the ever-present item with null key.
 	 *
+	 *  Use @a empty() if you only want to know if the table is empty or
+	 *  not.
+	 *
 	 *  @return The number of entries in the table.
 	 */
 	flint_tablesize_t get_entry_count() const {
 	    return item_count;
 	}
 
+	/// Return true if there are no entries in the table.
+	bool empty() const {
+	    // Prior to 1.1.4/1.0.18, item_count was stored in 32 bits, so we
+	    // can't trust it as there could be more than 1<<32 entries.
+	    //
+	    // In theory it should wrap, so if non-zero the table isn't empty,
+	    // but the table this was first noticed in wasn't off by a multiple
+	    // of 1<<32.
+
+	    // An empty table will always have level == 0, and most non-empty
+	    // tables will have more levels, so use that as a short-cut.
+	    return (level == 0) && really_empty();
+	}
+
 	/** Get a cursor for reading from the table.
 	 *
 	 *  The cursor is owned by the caller - it is the caller's
