@@ -3,7 +3,7 @@
  * with revision 1, which makes it especially fast to search.
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -578,5 +578,8 @@ main(int argc, char **argv)
     } catch (const Xapian::Error &error) {
 	cerr << argv[0] << ": " << error.get_description() << endl;
 	exit(1);
+    } catch (const char * msg) {
+	cerr << argv[0] << ": " << msg << endl;
+	exit(1);
     }
 }
