@@ -78,7 +78,7 @@ show_db_stats(Database &db)
 	    ++terms;
 	    ++t;
 	}
-	cout << "number of unique terms = " << terms << endl;
+	cout << "number of distinct terms = " << terms << endl;
     }
 }
 
