@@ -21,17 +21,17 @@
  * USA
  */
 
-// NB don't prune - even with one sublist we still translate docids...
-
 #include <config.h>
 #include "multimatch.h"
 #include "emptypostlist.h"
 #include "mergepostlist.h"
 #include "branchpostlist.h"
+#include "omassert.h"
 #include "omdebug.h"
-#include <xapian/enquire.h>
 #include <xapian/errorhandler.h>
 
+// NB don't prune - even with one sublist we still translate docids...
+
 MergePostList::MergePostList(std::vector<PostList *> plists_,
 			     MultiMatch *matcher_,
 			     Xapian::ErrorHandler * errorhandler_)
@@ -56,7 +56,7 @@ MergePostList::next(Xapian::weight w_min)
     DEBUGCALL(MATCH, PostList *, "MergePostList::next", w_min);
     DEBUGLINE(MATCH, "current = " << current);
     if (current == -1) current = 0;
-    do {
+    while (true) {
 	// FIXME: should skip over Remote matchers which aren't ready yet
 	// and come back to them later...
 	try {
@@ -74,7 +74,9 @@ MergePostList::next(Xapian::weight w_min)
 		throw;
 	    }
 	}
-    } while (unsigned(current) < plists.size());
+	if (unsigned(current) >= plists.size()) break;
+	if (matcher) matcher->recalc_maxweight();
+    }
     DEBUGLINE(MATCH, "current = " << current);
     RETURN(NULL);
 }
