@@ -184,15 +184,15 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
 		       Xapian::valueno sort_key_,
 		       Xapian::Enquire::Internal::sort_setting sort_by_,
 		       bool sort_value_forward_,
-		       const Xapian::Sorter * sorter_,
 		       Xapian::ErrorHandler * errorhandler_,
 		       Stats & stats,
-		       const Xapian::Weight * weight_)
+		       const Xapian::Weight * weight_,
+		       bool have_sorter, bool have_mdecider)
 	: db(db_), query(query_),
 	  collapse_key(collapse_key_), percent_cutoff(percent_cutoff_),
 	  weight_cutoff(weight_cutoff_), order(order_),
 	  sort_key(sort_key_), sort_by(sort_by_),
-	  sort_value_forward(sort_value_forward_), sorter(sorter_),
+	  sort_value_forward(sort_value_forward_),
 	  errorhandler(errorhandler_), weight(weight_),
 	  is_remote(db.internal.size())
 {
@@ -202,8 +202,8 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
 	      percent_cutoff_ << ", " << weight_cutoff_ << ", " <<
 	      int(order_) << ", " << sort_key_ << ", " <<
 	      int(sort_by_) << ", " << sort_value_forward_ << ", " <<
-	      sorter_ << ", " <<
-	      errorhandler_ << ", " << stats << ", [weight_]");
+	      errorhandler_ << ", " << stats << ", [weight_], " <<
+	      have_sorter << ", " << have_mdecider);
 
     if (!query) return;
     query->validate_query();
@@ -221,9 +221,12 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 	    RemoteDatabase *rem_db = subdb->as_remotedatabase();
 	    if (rem_db) {
-		if (sorter) {
+		if (have_sorter) {
 		    throw Xapian::UnimplementedError("Xapian::Sorter not supported for the remote backend");
 		}
+		if (have_mdecider) {
+		    throw Xapian::UnimplementedError("Xapian::MatchDecider not supported for the remote backend");
+		}
 		rem_db->set_query(query, qlen, collapse_key, order, sort_key,
 				  sort_by, sort_value_forward, percent_cutoff,
 				  weight_cutoff, weight, subrsets[i]);
@@ -291,7 +294,8 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		     Xapian::MSet & mset,
 		     const Stats & stats,
 		     const Xapian::MatchDecider *mdecider,
-		     const Xapian::MatchDecider *matchspy)
+		     const Xapian::MatchDecider *matchspy,
+		     const Xapian::Sorter *sorter)
 {
     DEBUGCALL(MATCH, void, "MultiMatch::get_mset", first << ", " << maxitems
 	      << ", " << check_at_least << ", ...");
