@@ -168,6 +168,7 @@ TcpServer::get_listening_socket(const std::string & host, int port
     }
 
     struct sockaddr_in addr;
+    memset(&addr, 0, sizeof(addr));
     addr.sin_family = AF_INET;
     addr.sin_port = htons(port);
     if (host.empty()) {
@@ -192,7 +193,7 @@ TcpServer::get_listening_socket(const std::string & host, int port
 		);
 	}
 
-	memcpy(&addr.sin_addr, hostent->h_addr, sizeof(addr.sin_addr));
+	memcpy(&addr.sin_addr, hostent->h_addr, hostent->h_length);
     }
 
     retval = bind(socketfd,
