@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,7 +34,6 @@
 #endif
 
 #include <algorithm>
-#include <iomanip>
 #include <iostream>
 
 #ifdef HAVE_STREAMBUF
@@ -88,7 +87,8 @@ const char * expected_exception = NULL;
 std::ostringstream tout;
 
 int test_driver::runs = 0;
-test_driver::result test_driver::total = {0, 0, 0};
+test_driver::result test_driver::subtotal;
+test_driver::result test_driver::total;
 string test_driver::argv0;
 string test_driver::opt_help;
 map<int, string *> test_driver::short_opts;
@@ -98,6 +98,16 @@ string test_driver::col_red, test_driver::col_green;
 string test_driver::col_yellow, test_driver::col_reset;
 bool test_driver::use_cr = false;
 
+void
+test_driver::write_and_clear_tout()
+{
+    const string & s = tout.str();
+    if (!s.empty()) {
+	out << '\n' << s;
+	tout.str(string());
+    }
+}
+
 string
 test_driver::get_srcdir()
 {
@@ -222,7 +232,7 @@ test_driver::runtest(const test_desc *test)
 #endif
 
     while (true) {
-	tout.str("");
+	tout.str(string());
 	SignalRedirector sig; // use object so signal handlers are reset
 	if (!setjmp(jb)) {
 	    static bool catch_signals =
@@ -243,13 +253,8 @@ test_driver::runtest(const test_desc *test)
 		}
 #endif
 		if (!test->run()) {
-		    string s = tout.str();
-		    if (!s.empty()) {
-			out << '\n' << tout.str();
-			if (s[s.size() - 1] != '\n') out << endl;
-			tout.str("");
-		    }
-		    out << " " << col_red << "FAILED" << col_reset;
+		    out << col_red << " FAILED" << col_reset;
+		    write_and_clear_tout();
 		    return FAIL;
 		}
 #ifdef HAVE_VALGRIND
@@ -258,7 +263,7 @@ test_driver::runtest(const test_desc *test)
 		    // leak checks, otherwise the buffers holding the output
 		    // may be identified as a memory leak (especially if >1K of
 		    // output has been buffered it appears...)
-		    tout.str("");
+		    tout.str(string());
 #define REPORT_FAIL_VG(M) do { \
     if (verbose) { \
 	while (true) { \
@@ -313,9 +318,9 @@ test_driver::runtest(const test_desc *test)
 		    }
 		    lseek(vg_log_fd, curpos, SEEK_SET);
 
-		    VALGRIND_DO_LEAK_CHECK;
 		    int vg_errs2 = VALGRIND_COUNT_ERRORS;
 		    vg_errs = vg_errs2 - vg_errs;
+		    VALGRIND_DO_LEAK_CHECK;
 		    long vg_leaks2 = 0, vg_dubious2 = 0, vg_reachable2 = 0;
 		    long dummy;
 		    VALGRIND_COUNT_LEAKS(vg_leaks2, vg_dubious2, vg_reachable2,
@@ -340,7 +345,7 @@ test_driver::runtest(const test_desc *test)
 			// then we need to rerun the test to see if the leak is
 			// real...
 			if (runcount == 0) {
-			    out << " " << col_yellow << "PROBABLY LEAKED MEMORY - RETRYING TEST" << col_reset;
+			    out << col_yellow << " PROBABLY LEAKED MEMORY - RETRYING TEST" << col_reset;
 			    ++runcount;
 			    continue;
 			}
@@ -348,10 +353,11 @@ test_driver::runtest(const test_desc *test)
 			return FAIL;
 		    }
 		    if (vg_reachable > 0) {
-			// FIXME:
 			// C++ STL implementations often "horde" released
-			// memory - perhaps we can supply our own allocator
-			// so we can tell the difference?
+			// memory - for GCC 3.4 and newer the runtest script
+			// sets GLIBCXX_FORCE_NEW=1 which will disable this
+			// behaviour so we avoid this issue, but for older
+			// GCC and other compilers this may be an issue.
 			//
 			// See also:
 			// http://valgrind.org/docs/FAQ/#faq.reports
@@ -360,7 +366,7 @@ test_driver::runtest(const test_desc *test)
 			// if more is leaked - hopefully this shouldn't give
 			// false positives.
 			if (runcount == 0) {
-			    out << " " << col_yellow << "POSSIBLE UNRELEASED MEMORY - RETRYING TEST" << col_reset;
+			    out << col_yellow << " POSSIBLE UNRELEASED MEMORY - RETRYING TEST" << col_reset;
 			    ++runcount;
 			    continue;
 			}
@@ -370,61 +376,27 @@ test_driver::runtest(const test_desc *test)
 		}
 #endif
 	    } catch (const TestFail &) {
-		string s = tout.str();
-		if (!s.empty()) {
-		    out << '\n' << tout.str();
-		    if (s[s.size() - 1] != '\n') out << endl;
-		    tout.str("");
-		}
-		out << " " << col_red << "FAILED" << col_reset;
+		out << col_red << " FAILED" << col_reset;
+		write_and_clear_tout();
 		return FAIL;
 	    } catch (const TestSkip &) {
-		string s = tout.str();
-		if (!s.empty()) {
-		    out << '\n' << tout.str();
-		    if (s[s.size() - 1] != '\n') out << endl;
-		    tout.str("");
-		}
-		out << " " << col_yellow << "SKIPPED" << col_reset;
+		out << col_yellow << " SKIPPED" << col_reset;
+		write_and_clear_tout();
 		return SKIP;
 	    } catch (const Xapian::Error &err) {
 		string errclass = err.get_type();
 		if (expected_exception && expected_exception == errclass) {
-		    out << " " << col_yellow << "C++ FAILED TO CATCH " << errclass << col_reset;
+		    out << col_yellow << " C++ FAILED TO CATCH " << errclass << col_reset;
 		    return SKIP;
 		}
-		string s = tout.str();
-		if (!s.empty()) {
-		    out << '\n' << tout.str();
-		    if (s[s.size() - 1] != '\n') out << endl;
-		    tout.str("");
-		}
-		out << " " << col_red << errclass << col_reset;
-		if (verbose) out << err.get_description() << endl;
+		out << " " << col_red << err.get_description() << col_reset;
+		write_and_clear_tout();
 		return FAIL;
 	    } catch (const string & msg) {
-		string s = tout.str();
-		if (!s.empty()) {
-		    out << '\n' << tout.str();
-		    if (s[s.size() - 1] != '\n') out << endl;
-		    tout.str("");
-		}
-		out << " " << col_red << "EXCEPTION: ";
-		size_t cutoff = min(size_t(40), msg.size());
-		cutoff = find(msg.begin(), msg.begin() + cutoff, '\n') - msg.begin();
-		if (verbose || cutoff == msg.size())
-		    out << msg;
-		else
-		    out << msg.substr(0, cutoff) << "...";
-		out << col_reset;
+		out << col_red << " EXCEPTION std::string " << msg << col_reset;
+		write_and_clear_tout();
 		return FAIL;
 	    } catch (const std::exception & e) {
-		string s = tout.str();
-		if (!s.empty()) {
-		    out << '\n' << tout.str();
-		    if (s[s.size() - 1] != '\n') out << endl;
-		    tout.str("");
-		}
 		out << " " << col_red;
 #ifndef USE_RTTI
 		out << "std::exception";
@@ -445,43 +417,33 @@ test_driver::runtest(const test_desc *test)
 		out << name;
 # endif
 #endif
-		out << ": " << e.what();
-		out << col_reset;
+		out << ": " << e.what() << col_reset;
+		write_and_clear_tout();
 		return FAIL;
 	    } catch (...) {
-		string s = tout.str();
-		if (!s.empty()) {
-		    out << '\n' << tout.str();
-		    if (s[s.size() - 1] != '\n') out << endl;
-		    tout.str("");
-		}
-		out << " " << col_red << "UNKNOWN EXCEPTION" << col_reset;
+		out << col_red << " UNKNOWN EXCEPTION" << col_reset;
+		write_and_clear_tout();
 		return FAIL;
 	    }
-	} else {
-	    // caught signal
-	    string s = tout.str();
-	    if (!s.empty()) {
-		out << '\n' << tout.str();
-		if (s[s.size() - 1] != '\n') out << endl;
-		tout.str("");
-	    }
-	    const char *signame = "SIGNAL";
-	    switch (signum) {
-		case SIGSEGV: signame = "SIGSEGV"; break;
-		case SIGFPE: signame = "SIGFPE"; break;
-		case SIGILL: signame = "SIGILL"; break;
+	    return PASS;
+	}
+
+	// Caught a signal.
+	const char *signame = "SIGNAL";
+	switch (signum) {
+	    case SIGSEGV: signame = "SIGSEGV"; break;
+	    case SIGFPE: signame = "SIGFPE"; break;
+	    case SIGILL: signame = "SIGILL"; break;
 #ifdef SIGBUS
-		case SIGBUS: signame = "SIGBUS"; break;
+	    case SIGBUS: signame = "SIGBUS"; break;
 #endif
 #ifdef SIGSTKFLT
-		case SIGSTKFLT: signame = "SIGSTKFLT"; break;
+	    case SIGSTKFLT: signame = "SIGSTKFLT"; break;
 #endif
-	    }
-	    out << " " << col_red << signame << col_reset;
-	    return FAIL;
 	}
-	return PASS;
+	out << " " << col_red << signame << col_reset;
+	write_and_clear_tout();
+	return FAIL;
     }
 }
 
@@ -506,7 +468,7 @@ test_driver::do_run_tests(vector<string>::const_iterator b,
     set<string> m(b, e);
     bool check_name = !m.empty();
 
-    test_driver::result res = {0, 0, 0};
+    test_driver::result res;
 
     for (const test_desc *test = tests; test->name; test++) {
 	bool do_this_test = !check_name;
@@ -563,9 +525,22 @@ test_driver::usage()
     exit(1);
 }
 
+/* Needs C linkage so we can pass it to atexit() without problems. */
+extern "C" {
+// Call upon program exit if there's more than one test run.
+static void
+report_totals(void)
+{
+    test_driver::report(test_driver::total, "total");
+}
+}
+
 void
 test_driver::report(const test_driver::result &r, const string &desc)
 {
+    // Report totals at the end if we reported two or more subtotals.
+    if (++runs == 2) atexit(report_totals);
+
     if (r.succeeded != 0 || r.failed != 0) {
 	cout << argv0 << " " << desc << ": ";
 
@@ -586,23 +561,13 @@ test_driver::report(const test_driver::result &r, const string &desc)
     }
 }
 
-/* Needs C linkage so we can pass it to atexit() without problems. */
-extern "C" {
-// Call upon program exit if there's more than one test run.
-static void
-report_totals(void)
-{
-    test_driver::report(test_driver::total, "total");
-}
-}
-
 void
 test_driver::add_command_line_option(const string &l, char s, string * arg)
 {
     short_opts.insert(make_pair<int, string *>(int(s), arg));
     opt_help += "[-";
     opt_help += s;
-    opt_help += '=';
+    opt_help += ' ';
     opt_help += l;
     opt_help += "] ";
 }
@@ -629,23 +594,23 @@ test_driver::parse_command_line(int argc, char **argv)
     }
 #endif
 
-    struct option long_opts[] = {
+    const struct option long_opts[] = {
 	{"verbose",		no_argument, 0, 'v'},
 	{"abort-on-error",	no_argument, 0, 'o'},
 	{"help",		no_argument, 0, 'h'},
 	{NULL,			0, 0, 0}
     };
 
-    string opts = "voh";
+    string short_opts_string = "voh";
     map<int, string *>::const_iterator i;
     for (i = short_opts.begin(); i != short_opts.end(); ++i) {
-	opts += char(i->first);
-	opts += ':';
+	short_opts_string += char(i->first);
+	short_opts_string += ':';
     }
+    const char * opts = short_opts_string.c_str();
 
     int c;
-    while ((c = gnu_getopt_long(argc, argv, opts.c_str(), long_opts, 0)) != EOF)
-    {
+    while ((c = gnu_getopt_long(argc, argv, opts, long_opts, 0)) != -1) {
 	switch (c) {
 	    case 'v':
 		verbose = true;
@@ -700,21 +665,12 @@ test_driver::parse_command_line(int argc, char **argv)
 int
 test_driver::run(const test_desc *tests)
 {
-    // If we're running more than one batch of tests, report the totals
-    // at the end.
-    if (runs == 1) atexit(report_totals);
-    ++runs;
-
     test_driver driver(tests);
 
     test_driver::result myresult;
     myresult = driver.run_tests(test_names.begin(), test_names.end());
 
-    report(myresult, "completed test run");
-
-    total.succeeded += myresult.succeeded;
-    total.failed += myresult.failed;
-    total.skipped += myresult.skipped;
+    subtotal += myresult;
 
     return bool(myresult.failed); // if 0, then everything passed
 }
